@@ -19,6 +19,12 @@
 #include "AuthMethodList.h"
 #include "include/types.h"
 
+// Different classes of session crypto handling
+
+#define SESSION_CRYPTO_NONE 0
+#define SESSION_SYMMETRIC_AUTHENTICATE 1
+#define SESSION_SYMMETRIC_ENCRYPT 2
+
 class CephContext;
 class KeyRing;
 class RotatingKeyRing;
@@ -28,18 +34,18 @@ struct AuthAuthorizeHandler {
   virtual bool verify_authorizer(CephContext *cct, KeyStore *keys,
 				 bufferlist& authorizer_data, bufferlist& authorizer_reply,
                                  EntityName& entity_name, uint64_t& global_id,
-				 AuthCapsInfo& caps_info, uint64_t *auid = NULL) = 0;
+				 AuthCapsInfo& caps_info, CryptoKey& session_key, uint64_t *auid = NULL) = 0;
+  virtual int authorizer_session_crypto() = 0;
 };
 
 class AuthAuthorizeHandlerRegistry {
   Mutex m_lock;
   map<int,AuthAuthorizeHandler*> m_authorizers;
-  CephContext *cct;
   AuthMethodList supported;
 
 public:
   AuthAuthorizeHandlerRegistry(CephContext *cct_, std::string methods)
-    : m_lock("AuthAuthorizeHandlerRegistry::m_lock"), cct(cct_), supported(cct_, methods)
+    : m_lock("AuthAuthorizeHandlerRegistry::m_lock"), supported(cct_, methods)
   {}
   ~AuthAuthorizeHandlerRegistry();
   
