@@ -42,7 +42,7 @@ int get_random_bytes(char *buf, int len)
   if (fd < 0)
     return -errno;
   int ret = safe_read_exact(fd, buf, len);
-  TEMP_FAILURE_RETRY(::close(fd));
+  VOID_TEMP_FAILURE_RETRY(::close(fd));
   return ret;
 }
 
@@ -373,9 +373,7 @@ void CryptoKey::decrypt(CephContext *cct, const bufferlist& in, bufferlist& out,
 
 void CryptoKey::print(std::ostream &out) const
 {
-  string a;
-  encode_base64(a);
-  out << a;
+  out << encode_base64();
 }
 
 void CryptoKey::to_str(std::string& s) const
@@ -385,3 +383,16 @@ void CryptoKey::to_str(std::string& s) const
   hex2str(secret.c_str(), secret.length(), buf, len);
   s = buf;
 }
+
+void CryptoKey::encode_formatted(string label, Formatter *f, bufferlist &bl)
+{
+  f->open_object_section(label.c_str());
+  f->dump_string("key", encode_base64());
+  f->close_section();
+  f->flush(bl);
+}
+
+void CryptoKey::encode_plaintext(bufferlist &bl)
+{
+  bl.append(encode_base64());
+}
