@@ -50,6 +50,10 @@ check_host() {
 
     #echo host for $name is $host, i am $hostname
 
+    if [ -e "/var/lib/ceph/$type/ceph-$id/upstart" ]; then
+	return 1
+    fi
+
     # sysvinit managed instance in standard location?
     if [ -e "/var/lib/ceph/$type/ceph-$id/sysvinit" ]; then
 	host="$hostname"
@@ -136,7 +140,7 @@ do_root_cmd() {
 get_local_daemon_list() {
     type=$1
     if [ -d "/var/lib/ceph/$type" ]; then
-	for i in `find /var/lib/ceph/$type -mindepth 1 -maxdepth 1 -type d -printf '%f\n'`; do
+	for i in `find -L /var/lib/ceph/$type -mindepth 1 -maxdepth 1 -type d -printf '%f\n'`; do
 	    if [ -e "/var/lib/ceph/$type/$i/sysvinit" ]; then
 		id=`echo $i | sed 's/[^-]*-//'`
 		local="$local $type.$id"
