@@ -255,8 +255,8 @@ public:
   void resend_unsafe_requests(MetaSession *s);
 
   // mds requests
-  tid_t last_tid, last_flush_seq;
-  map<tid_t, MetaRequest*> mds_requests;
+  ceph_tid_t last_tid, last_flush_seq;
+  map<ceph_tid_t, MetaRequest*> mds_requests;
   set<int>                 failed_mds;
 
   void dump_mds_requests(Formatter *f);
@@ -445,7 +445,7 @@ protected:
   void add_update_cap(Inode *in, MetaSession *session, uint64_t cap_id,
 		      unsigned issued, unsigned seq, unsigned mseq, inodeno_t realm,
 		      int flags);
-  void remove_cap(Cap *cap);
+  void remove_cap(Cap *cap, bool queue_release);
   void remove_all_caps(Inode *in);
   void remove_session_caps(MetaSession *session);
   void mark_caps_dirty(Inode *in, int caps);
@@ -547,6 +547,8 @@ private:
   void _closedir(dir_result_t *dirp);
 
   // other helpers
+  void _fragmap_remove_non_leaves(Inode *in);
+
   void _ll_get(Inode *in);
   int _ll_put(Inode *in, int num);
   void _ll_drop_pins();
@@ -554,6 +556,19 @@ private:
   Fh *_create_fh(Inode *in, int flags, int cmode);
   int _release_fh(Fh *fh);
 
+
+  struct C_Readahead : public Context {
+    Client *client;
+    Inode *inode;
+    C_Readahead(Client *c, Inode *i)
+      : client(c),
+	inode(i) { }
+    void finish(int r) {
+      lsubdout(client->cct, client, 20) << "C_Readahead on " << inode << dendl;
+      client->put_inode(inode, 1);
+    }
+  };
+
   int _read_sync(Fh *f, uint64_t off, uint64_t len, bufferlist *bl);
   int _read_async(Fh *f, uint64_t off, uint64_t len, bufferlist *bl);
 
@@ -680,7 +695,9 @@ public:
   int open(const char *path, int flags, mode_t mode=0);
   int open(const char *path, int flags, mode_t mode, int stripe_unit, int stripe_count, int object_size, const char *data_pool);
   int lookup_hash(inodeno_t ino, inodeno_t dirino, const char *name);
-  int lookup_ino(inodeno_t ino);
+  int lookup_ino(inodeno_t ino, Inode **inode=NULL);
+  int lookup_parent(Inode *in, Inode **parent=NULL);
+  int lookup_name(Inode *in, Inode *parent);
   int close(int fd);
   loff_t lseek(int fd, loff_t offset, int whence);
   int read(int fd, char *buf, loff_t size, loff_t offset=-1);
