@@ -137,7 +137,7 @@ bool Inode::put_cap_ref(int cap)
 
 bool Inode::is_any_caps()
 {
-  return caps.size() || exporting_mds >= 0;
+  return caps.size();
 }
 
 bool Inode::cap_is_valid(Cap* cap)
@@ -155,7 +155,7 @@ bool Inode::cap_is_valid(Cap* cap)
 
 int Inode::caps_issued(int *implemented)
 {
-  int c = exporting_issued | snap_caps;
+  int c = snap_caps;
   int i = 0;
   for (map<int,Cap*>::iterator it = caps.begin();
        it != caps.end();
@@ -183,7 +183,7 @@ void Inode::try_touch_cap(int mds)
 
 bool Inode::caps_issued_mask(unsigned mask)
 {
-  int c = exporting_issued | snap_caps;
+  int c = snap_caps;
   if ((c & mask) == mask)
     return true;
   // prefer auth cap
@@ -390,10 +390,6 @@ void Inode::dump(Formatter *f) const
     f->dump_int("snap_caps", snap_caps);
     f->dump_int("snap_cap_refs", snap_cap_refs);
   }
-  if (exporting_issued || exporting_mseq) {
-    f->dump_stream("exporting_issued") << ccap_string(exporting_issued);
-    f->dump_int("exporting_mseq", exporting_mds);
-  }
 
   f->dump_stream("hold_caps_until") << hold_caps_until;
   f->dump_unsigned("last_flush_tid", last_flush_tid);
