@@ -25,6 +25,8 @@
  * parameters as the client sees them - it would be silly to mention
  * in each one that they take an input and an output bufferlist.
  */
+#include "include/int_types.h"
+#include "include/types.h"
 
 #include <algorithm>
 #include <cstring>
@@ -35,10 +37,8 @@
 #include <sstream>
 #include <vector>
 
-#include "include/types.h"
 #include "objclass/objclass.h"
 #include "include/rbd_types.h"
-#include <inttypes.h>
 
 #include "cls/rbd/cls_rbd.h"
 
@@ -92,7 +92,6 @@ cls_method_handle_t h_dir_rename_image;
 cls_method_handle_t h_old_snapshots_list;
 cls_method_handle_t h_old_snapshot_add;
 cls_method_handle_t h_old_snapshot_remove;
-cls_method_handle_t h_assign_bid;
 
 #define RBD_MAX_KEYS_READ 64
 #define RBD_SNAP_KEY_PREFIX "snapshot_"
@@ -678,7 +677,7 @@ int set_stripe_unit_count(cls_method_context_t hctx, bufferlist *in, bufferlist
     CLS_ERR("failed to read the order off of disk: %s", strerror(r));
     return r;
   }
-  if ((1ull << order) % stripe_unit) {
+  if ((1ull << order) % stripe_unit || stripe_unit > (1ull << order)) {
     CLS_ERR("stripe unit %llu is not a factor of the object size %llu",
             (unsigned long long)stripe_unit, 1ull << order);
     return -EINVAL;
@@ -1526,7 +1525,8 @@ static int dir_remove_image_helper(cls_method_context_t hctx,
   string id_key = dir_key_for_id(id);
   int r = read_key(hctx, name_key, &stored_id);
   if (r < 0) {
-    CLS_ERR("error reading name to id mapping: %d", r);
+    if (r != -ENOENT)
+      CLS_ERR("error reading name to id mapping: %d", r);
     return r;
   }
   r = read_key(hctx, id_key, &stored_name);
@@ -1620,7 +1620,8 @@ int dir_get_id(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
   string id;
   int r = read_key(hctx, dir_key_for_name(name), &id);
   if (r < 0) {
-    CLS_ERR("error reading id for name '%s': %d", name.c_str(), r);
+    if (r != -ENOENT)
+      CLS_ERR("error reading id for name '%s': %d", name.c_str(), r);
     return r;
   }
   ::encode(id, *out);
@@ -1980,48 +1981,6 @@ int old_snapshot_remove(cls_method_context_t hctx, bufferlist *in, bufferlist *o
 }
 
 
-/* assign block id. This method should be called on the rbd_info object */
-int rbd_assign_bid(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
-{
-  struct rbd_info info;
-  int rc;
-  bufferlist bl;
-
-  rc = cls_cxx_read(hctx, 0, sizeof(info), &bl);
-  if (rc < 0 && rc != -EEXIST)
-    return rc;
-
-  if (rc && rc < (int)sizeof(info)) {
-    CLS_ERR("bad rbd_info object, read %d bytes, expected %d", rc,
-	    (int)sizeof(info));
-    return -EIO;
-  }
-
-  uint64_t max_id;
-  if (rc) {
-    memcpy(&info, bl.c_str(), sizeof(info));
-    max_id = info.max_id + 1;
-    info.max_id = max_id;
-  } else {
-    memset(&info, 0, sizeof(info));
-    max_id = 0;
-  }
-
-  bufferlist newbl;
-  bufferptr bp(sizeof(info));
-  memcpy(bp.c_str(), &info, sizeof(info));
-  newbl.push_back(bp);
-  rc = cls_cxx_write_full(hctx, &newbl);
-  if (rc < 0) {
-    CLS_ERR("error writing rbd_info, got rc=%d", rc);
-    return rc;
-  }
-
-  ::encode(max_id, *out);
-
-  return out->length();
-}
-
 void __cls_init()
 {
   CLS_LOG(20, "Loaded rbd class!");
@@ -2132,10 +2091,5 @@ void __cls_init()
 			  CLS_METHOD_RD | CLS_METHOD_WR,
 			  old_snapshot_remove, &h_old_snapshot_remove);
 
-  /* assign a unique block id for rbd blocks */
-  cls_register_cxx_method(h_class, "assign_bid",
-			  CLS_METHOD_RD | CLS_METHOD_WR,
-			  rbd_assign_bid, &h_assign_bid);
-
   return;
 }
