@@ -44,6 +44,9 @@ class Formatter {
   virtual void dump_int(const char *name, int64_t s) = 0;
   virtual void dump_float(const char *name, double d) = 0;
   virtual void dump_string(const char *name, std::string s) = 0;
+  virtual void dump_bool(const char *name, bool b) {
+    dump_format_unquoted(name, "%s", (b ? "true" : "false"));
+  }
   virtual std::ostream& dump_stream(const char *name) = 0;
   virtual void dump_format(const char *name, const char *fmt, ...) = 0;
   virtual void dump_format_unquoted(const char *name, const char *fmt, ...) = 0;
@@ -62,7 +65,7 @@ class Formatter {
   }
 };
 
-Formatter *new_formatter(const std::string type);
+Formatter *new_formatter(const std::string &type);
 
 class JSONFormatter : public Formatter {
  public:
