@@ -78,20 +78,24 @@ public:
 
   int signal_exit(int r) {
     if (forked) {
-      // tell parent
-      (void)::write(fd[1], &r, sizeof(r));
+      // tell parent.  this shouldn't fail, but if it does, pass the
+      // error back to the parent.
+      int ret = safe_write(fd[1], &r, sizeof(r));
+      if (ret <= 0)
+	return ret;
     }
     return r;
   }
   void exit(int r) {
     signal_exit(r);
-    exit(r);
+    ::exit(r);
   }
 
   void daemonize() {
     assert(forked);
     static int r = -1;
-    (void)::write(fd[1], &r, sizeof(r));
+    int r2 = ::write(fd[1], &r, sizeof(r));
+    r += r2;  // make the compiler shut up about the unused return code from ::write(2).
   }
   
 };
