@@ -17,13 +17,13 @@
 #include <stdint.h>
 #include <include/utime.h>
 #include "common/Mutex.h"
-#include "include/histogram.h"
+#include "common/histogram.h"
 #include "include/xlist.h"
 #include "msg/Message.h"
-#include <tr1/memory>
+#include "include/memory.h"
 
 class TrackedOp;
-typedef std::tr1::shared_ptr<TrackedOp> TrackedOpRef;
+typedef ceph::shared_ptr<TrackedOp> TrackedOpRef;
 
 class OpTracker;
 class OpHistory {
@@ -67,9 +67,11 @@ class OpTracker {
   int log_threshold;
 
 public:
+  bool tracking_enabled;
   CephContext *cct;
-  OpTracker(CephContext *cct_) : seq(0), ops_in_flight_lock("OpTracker mutex"),
-      complaint_time(0), log_threshold(0), cct(cct_) {}
+  OpTracker(CephContext *cct_, bool tracking) : seq(0), ops_in_flight_lock("OpTracker mutex"),
+						complaint_time(0), log_threshold(0),
+						tracking_enabled(tracking), cct(cct_) {}
   void set_complaint_and_threshold(float time, int threshold) {
     complaint_time = time;
     log_threshold = threshold;
