@@ -49,6 +49,7 @@ public:
       : cct(cct), hb(hb), grace(grace), suicide_grace(suicide_grace) {}
   public:
     void reset_tp_timeout();
+    void suspend_tp_timeout();
   };
 private:
 
@@ -153,7 +154,7 @@ public:
     }
 
   };
-  template<typename T, typename U>
+  template<typename T, typename U = T>
   class WorkQueueVal : public WorkQueue_ {
     Mutex _lock;
     ThreadPool *pool;
@@ -389,6 +390,43 @@ public:
   void drain(WorkQueue_* wq = 0);
 };
 
+class GenContextWQ :
+  public ThreadPool::WorkQueueVal<GenContext<ThreadPool::TPHandle&>*> {
+  list<GenContext<ThreadPool::TPHandle&>*> _queue;
+public:
+  GenContextWQ(const string &name, time_t ti, ThreadPool *tp)
+    : ThreadPool::WorkQueueVal<
+      GenContext<ThreadPool::TPHandle&>*>(name, ti, ti*10, tp) {}
+  
+  void _enqueue(GenContext<ThreadPool::TPHandle&> *c) {
+    _queue.push_back(c);
+  };
+  void _enqueue_front(GenContext<ThreadPool::TPHandle&> *c) {
+    _queue.push_front(c);
+  }
+  bool _empty() {
+    return _queue.empty();
+  }
+  GenContext<ThreadPool::TPHandle&> *_dequeue() {
+    assert(!_queue.empty());
+    GenContext<ThreadPool::TPHandle&> *c = _queue.front();
+    _queue.pop_front();
+    return c;
+  }
+  void _process(GenContext<ThreadPool::TPHandle&> *c, ThreadPool::TPHandle &tp) {
+    c->complete(tp);
+  }
+};
 
+class C_QueueInWQ : public Context {
+  GenContextWQ *wq;
+  GenContext<ThreadPool::TPHandle&> *c;
+public:
+  C_QueueInWQ(GenContextWQ *wq, GenContext<ThreadPool::TPHandle &> *c)
+    : wq(wq), c(c) {}
+  void finish(int) {
+    wq->queue(c);
+  }
+};
 
 #endif
