@@ -3,6 +3,7 @@
 #include "common/crc32c_intel_baseline.h"
 
 extern unsigned int crc32_iscsi_00(unsigned char const *buffer, int len, unsigned int crc);
+extern unsigned int crc32_iscsi_zero_00(unsigned char const *buffer, int len, unsigned int crc);
 
 #ifdef HAVE_GOOD_YASM_ELF64
 
@@ -11,6 +12,10 @@ uint32_t ceph_crc32c_intel_fast(uint32_t crc, unsigned char const *buffer, unsig
 	uint32_t v;
 	unsigned left;
 
+
+	if (!buffer)
+		return crc32_iscsi_zero_00(buffer, len, crc);
+
 	/*
 	 * the crc32_iscsi_00 method reads past buffer+len (because it
 	 * reads full words) which makes valgrind unhappy.  don't do
