@@ -53,8 +53,8 @@ int pipe_cloexec(int pipefd[2])
 	return 0;
 
 out:
-	TEMP_FAILURE_RETRY(close(pipefd[0]));
-	TEMP_FAILURE_RETRY(close(pipefd[1]));
+	VOID_TEMP_FAILURE_RETRY(close(pipefd[0]));
+	VOID_TEMP_FAILURE_RETRY(close(pipefd[1]));
 
 	return ret;
 #endif
