@@ -6,6 +6,8 @@
 #include <assert.h>
 #include <errno.h>
 
+#include "include/int_types.h"
+
 #include "builder.h"
 #include "hash.h"
 
@@ -24,6 +26,7 @@ struct crush_map *crush_create()
 	m->choose_local_fallback_tries = 5;
 	m->choose_total_tries = 19;
 	m->chooseleaf_descend_once = 0;
+	m->chooseleaf_vary_r = 0;
 	return m;
 }
 
