@@ -13,7 +13,6 @@
 #define CEPH_COMPAT_H
 
 #if defined(__FreeBSD__)
-#define	lseek64(fd, offset, whence)	lseek(fd, offset, whence)
 #define	ENODATA	61
 #define	MSG_MORE 0
 #endif /* !__FreeBSD__ */
@@ -27,4 +26,16 @@
   __result; })
 #endif
 
+#ifdef __cplusplus
+# define VOID_TEMP_FAILURE_RETRY(expression) \
+   static_cast<void>(TEMP_FAILURE_RETRY(expression))
+#else
+# define VOID_TEMP_FAILURE_RETRY(expression) \
+   do { (void)TEMP_FAILURE_RETRY(expression); } while (0)
+#endif
+
+#if defined(__FreeBSD__) || defined(__APPLE__)
+#define lseek64(fd, offset, whence) lseek(fd, offset, whence)
+#endif
+
 #endif /* !CEPH_COMPAT_H */
