@@ -102,7 +102,9 @@ int64_t librados::RadosClient::lookup_pool(const char *name)
 
   lock.Lock();
 
-  wait_for_osdmap();
+  int r = wait_for_osdmap();
+  if (r < 0)
+    return r;
   int64_t ret = osdmap.lookup_pg_pool_name(name);
   pool_cache_rwl.get_write();
   lock.Unlock();
@@ -120,6 +122,20 @@ int64_t librados::RadosClient::lookup_pool(const char *name)
   return ret;
 }
 
+bool librados::RadosClient::pool_requires_alignment(int64_t pool_id)
+{
+  Mutex::Locker l(lock);
+  return osdmap.have_pg_pool(pool_id) &&
+    osdmap.get_pg_pool(pool_id)->requires_aligned_append();
+}
+
+uint64_t librados::RadosClient::pool_required_alignment(int64_t pool_id)
+{
+  Mutex::Locker l(lock);
+  return osdmap.have_pg_pool(pool_id) ?
+    osdmap.get_pg_pool(pool_id)->required_alignment() : 0;
+}
+
 const char *librados::RadosClient::get_pool_name(int64_t pool_id)
 {
   Mutex::Locker l(lock);
@@ -129,7 +145,9 @@ const char *librados::RadosClient::get_pool_name(int64_t pool_id)
 int librados::RadosClient::pool_get_auid(uint64_t pool_id, unsigned long long *auid)
 {
   Mutex::Locker l(lock);
-  wait_for_osdmap();
+  int r = wait_for_osdmap();
+  if (r < 0)
+    return r;
   const pg_pool_t *pg = osdmap.get_pg_pool(pool_id);
   if (!pg)
     return -ENOENT;
@@ -140,7 +158,9 @@ int librados::RadosClient::pool_get_auid(uint64_t pool_id, unsigned long long *a
 int librados::RadosClient::pool_get_name(uint64_t pool_id, std::string *s)
 {
   Mutex::Locker l(lock);
-  wait_for_osdmap();
+  int r = wait_for_osdmap();
+  if (r < 0)
+    return r;
   const char *str = osdmap.get_pool_name(pool_id);
   if (!str)
     return -ENOENT;
@@ -214,7 +234,9 @@ int librados::RadosClient::connect()
   ldout(cct, 1) << "starting objecter" << dendl;
 
   err = -ENOMEM;
-  objecter = new Objecter(cct, messenger, &monclient, &osdmap, lock, timer);
+  objecter = new Objecter(cct, messenger, &monclient, &osdmap, lock, timer,
+			  cct->_conf->rados_mon_op_timeout,
+			  cct->_conf->rados_osd_op_timeout);
   if (!objecter)
     goto out;
   objecter->set_balanced_budget();
@@ -406,15 +428,34 @@ bool librados::RadosClient::_dispatch(Message *m)
   return true;
 }
 
-void librados::RadosClient::wait_for_osdmap()
+int librados::RadosClient::wait_for_osdmap()
 {
   assert(lock.is_locked());
+
+  utime_t timeout;
+  if (cct->_conf->rados_mon_op_timeout > 0)
+    timeout.set_from_double(cct->_conf->rados_mon_op_timeout);
+
   if (osdmap.get_epoch() == 0) {
     ldout(cct, 10) << __func__ << " waiting" << dendl;
-    while (osdmap.get_epoch() == 0)
-      cond.Wait(lock);
+    utime_t start = ceph_clock_now(cct);
+
+    while (osdmap.get_epoch() == 0) {
+      cond.WaitInterval(cct, lock, timeout);
+
+      utime_t elapsed = ceph_clock_now(cct) - start;
+      if (!timeout.is_zero() && elapsed > timeout)
+	break;
+    }
+
     ldout(cct, 10) << __func__ << " done waiting" << dendl;
+
+    if (osdmap.get_epoch() == 0) {
+      lderr(cct) << "timed out waiting for first osdmap from monitors" << dendl;
+      return -ETIMEDOUT;
+    }
   }
+  return 0;
 }
 
 int librados::RadosClient::wait_for_latest_osdmap()
@@ -438,7 +479,9 @@ int librados::RadosClient::wait_for_latest_osdmap()
 int librados::RadosClient::pool_list(std::list<std::string>& v)
 {
   Mutex::Locker l(lock);
-  wait_for_osdmap();
+  int r = wait_for_osdmap();
+  if (r < 0)
+    return r;
   for (map<int64_t,pg_pool_t>::const_iterator p = osdmap.get_pools().begin();
        p != osdmap.get_pools().end();
        ++p)
@@ -452,9 +495,11 @@ int librados::RadosClient::get_pool_stats(std::list<string>& pools,
   Mutex mylock("RadosClient::get_pool_stats::mylock");
   Cond cond;
   bool done;
+  int ret = 0;
 
   lock.Lock();
-  objecter->get_pool_stats(pools, &result, new C_SafeCond(&mylock, &cond, &done));
+  objecter->get_pool_stats(pools, &result, new C_SafeCond(&mylock, &cond, &done,
+							  &ret));
   lock.Unlock();
 
   mylock.Lock();
@@ -462,7 +507,7 @@ int librados::RadosClient::get_pool_stats(std::list<string>& pools,
     cond.Wait(mylock);
   mylock.Unlock();
 
-  return 0;
+  return ret;
 }
 
 int librados::RadosClient::get_fs_stats(ceph_statfs& stats)
@@ -470,15 +515,17 @@ int librados::RadosClient::get_fs_stats(ceph_statfs& stats)
   Mutex mylock ("RadosClient::get_fs_stats::mylock");
   Cond cond;
   bool done;
+  int ret = 0;
+
   lock.Lock();
-  objecter->get_fs_stats(stats, new C_SafeCond(&mylock, &cond, &done));
+  objecter->get_fs_stats(stats, new C_SafeCond(&mylock, &cond, &done, &ret));
   lock.Unlock();
 
   mylock.Lock();
   while (!done) cond.Wait(mylock);
   mylock.Unlock();
 
-  return 0;
+  return ret;
 }
 
 void librados::RadosClient::get() {
@@ -534,7 +581,9 @@ int librados::RadosClient::pool_create_async(string& name, PoolAsyncCompletionIm
 int librados::RadosClient::pool_delete(const char *name)
 {
   lock.Lock();
-  wait_for_osdmap();
+  int r = wait_for_osdmap();
+  if (r < 0)
+    return r;
   int tmp_pool_id = osdmap.lookup_pg_pool_name(name);
   if (tmp_pool_id < 0) {
     lock.Unlock();
@@ -563,13 +612,15 @@ int librados::RadosClient::pool_delete(const char *name)
 int librados::RadosClient::pool_delete_async(const char *name, PoolAsyncCompletionImpl *c)
 {
   Mutex::Locker l(lock);
-  wait_for_osdmap();
+  int r = wait_for_osdmap();
+  if (r < 0)
+    return r;
   int tmp_pool_id = osdmap.lookup_pg_pool_name(name);
   if (tmp_pool_id < 0)
     return -ENOENT;
 
   Context *onfinish = new C_PoolAsync_Safe(c);
-  int r = objecter->delete_pool(tmp_pool_id, onfinish);
+  r = objecter->delete_pool(tmp_pool_id, onfinish);
   if (r < 0) {
     delete onfinish;
   }
@@ -703,7 +754,7 @@ int librados::RadosClient::osd_command(int osd, vector<string>& cmd,
   Cond cond;
   bool done;
   int ret;
-  tid_t tid;
+  ceph_tid_t tid;
 
   if (osd < 0)
     return -EINVAL;
@@ -730,7 +781,7 @@ int librados::RadosClient::pg_command(pg_t pgid, vector<string>& cmd,
   Cond cond;
   bool done;
   int ret;
-  tid_t tid;
+  ceph_tid_t tid;
   lock.Lock();
   int r = objecter->pg_command(pgid, cmd, inbl, &tid, poutbl, prs,
 		        new C_SafeCond(&mylock, &cond, &done, &ret));
