@@ -2021,6 +2021,12 @@ extern "C" int rados_cluster_fsid(rados_t cluster, char *buf,
   return fsid.length();
 }
 
+extern "C" int rados_wait_for_latest_osdmap(rados_t cluster)
+{
+  librados::RadosClient *radosp = (librados::RadosClient *)cluster;
+  return radosp->wait_for_latest_osdmap();
+}
+
 extern "C" int rados_pool_list(rados_t cluster, char *buf, size_t len)
 {
   librados::RadosClient *client = (librados::RadosClient *)cluster;
@@ -2421,6 +2427,18 @@ extern "C" int rados_ioctx_pool_get_auid(rados_ioctx_t io, uint64_t *auid)
   return ctx->client->pool_get_auid(ctx->get_id(), (unsigned long long *)auid);
 }
 
+extern "C" int rados_ioctx_pool_requires_alignment(rados_ioctx_t io)
+{
+  librados::IoCtxImpl *ctx = (librados::IoCtxImpl *)io;
+  return ctx->client->pool_requires_alignment(ctx->get_id());
+}
+
+extern "C" uint64_t rados_ioctx_pool_required_alignment(rados_ioctx_t io)
+{
+  librados::IoCtxImpl *ctx = (librados::IoCtxImpl *)io;
+  return ctx->client->pool_required_alignment(ctx->get_id());
+}
+
 extern "C" void rados_ioctx_locator_set_key(rados_ioctx_t io, const char *key)
 {
   librados::IoCtxImpl *ctx = (librados::IoCtxImpl *)io;
@@ -3162,7 +3180,6 @@ extern "C" void rados_write_op_create(rados_write_op_t write_op,
 {
   ::ObjectOperation *oo = (::ObjectOperation *) write_op;
   if(category) {
-    std::string cpp_category = category;
     oo->create(exclusive, category);
   } else {
     oo->create(!!exclusive);
