@@ -3,7 +3,7 @@
 #ifndef CEPH_LIBRBD_IMAGECTX_H
 #define CEPH_LIBRBD_IMAGECTX_H
 
-#include <inttypes.h>
+#include "include/int_types.h"
 
 #include <map>
 #include <set>
@@ -69,6 +69,8 @@ namespace librbd {
     RWLock parent_lock; // protects parent_md and parent
     Mutex refresh_lock; // protects refresh_seq and last_refresh
 
+    unsigned extra_read_flags;
+
     bool old_format;
     uint8_t order;
     uint64_t size;
@@ -99,6 +101,7 @@ namespace librbd {
     void init_layout();
     void perf_start(std::string name);
     void perf_stop();
+    void set_read_flag(unsigned flag);
     int get_read_flags(librados::snap_t snap_id);
     int snap_set(std::string in_snap_name);
     void snap_unset();
@@ -136,7 +139,7 @@ namespace librbd {
     void flush_cache_aio(Context *onfinish);
     int flush_cache();
     void shutdown_cache();
-    int invalidate_cache();
+    void invalidate_cache();
     void clear_nonexistence_cache();
     int register_watch();
     void unregister_watch();
