@@ -11,9 +11,9 @@
  * Foundation.	See file COPYING.
  *
  */
+#include "include/int_types.h"
 
 #include <errno.h>
-#include <inttypes.h>
 
 #include "common/Cond.h"
 #include "common/dout.h"
@@ -514,12 +514,6 @@ namespace librbd {
     return librbd::aio_flush(ictx, (librbd::AioCompletion *)c->pc);
   }
 
-  int Image::invalidate_cache()
-  {
-    ImageCtx *ictx = (ImageCtx *)ctx;
-    return librbd::invalidate_cache(ictx);
-  }
-
 } // namespace librbd
 
 extern "C" void rbd_version(int *major, int *minor, int *extra)
@@ -1136,12 +1130,6 @@ extern "C" int rbd_aio_flush(rbd_image_t image, rbd_completion_t c)
   return librbd::aio_flush(ictx, (librbd::AioCompletion *)comp->pc);
 }
 
-extern "C" int rbd_invalidate_cache(rbd_image_t image)
-{
-  librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
-  return librbd::invalidate_cache(ictx);
-}
-
 extern "C" int rbd_aio_is_complete(rbd_completion_t c)
 {
   librbd::RBD::AioCompletion *comp = (librbd::RBD::AioCompletion *)c;
