@@ -48,26 +48,28 @@ class MExportCapsAck;
 
 class EImportStart;
 
+struct Mutation;
 
 class Migrator {
 private:
   MDS *mds;
   MDCache *cache;
 
-  uint64_t last_export_tid;
   // -- exports --
 public:
   // export stages.  used to clean up intelligently if there's a failure.
-  const static int EXPORT_CANCELLED     = 0;  // cancelled
-  const static int EXPORT_DISCOVERING   = 1;  // dest is disovering export dir
-  const static int EXPORT_FREEZING      = 2;  // we're freezing the dir tree
-  const static int EXPORT_PREPPING      = 3;  // sending dest spanning tree to export bounds
-  const static int EXPORT_WARNING       = 4;  // warning bystanders of dir_auth_pending
-  const static int EXPORT_EXPORTING     = 5;  // sent actual export, waiting for ack
-  const static int EXPORT_LOGGINGFINISH = 6;  // logging EExportFinish
-  const static int EXPORT_NOTIFYING     = 7;  // waiting for notifyacks
+  const static int EXPORT_CANCELLED	= 0;  // cancelled
+  const static int EXPORT_LOCKING	= 1;  // acquiring locks
+  const static int EXPORT_DISCOVERING	= 2;  // dest is disovering export dir
+  const static int EXPORT_FREEZING	= 3;  // we're freezing the dir tree
+  const static int EXPORT_PREPPING	= 4;  // sending dest spanning tree to export bounds
+  const static int EXPORT_WARNING	= 5;  // warning bystanders of dir_auth_pending
+  const static int EXPORT_EXPORTING	= 6;  // sent actual export, waiting for ack
+  const static int EXPORT_LOGGINGFINISH	= 7;  // logging EExportFinish
+  const static int EXPORT_NOTIFYING	= 8;  // waiting for notifyacks
   static const char *get_export_statename(int s) {
     switch (s) {
+    case EXPORT_LOCKING: return "locking";
     case EXPORT_DISCOVERING: return "discovering";
     case EXPORT_FREEZING: return "freezing";
     case EXPORT_PREPPING: return "prepping";
@@ -85,25 +87,23 @@ protected:
     int state;
     int peer;
     uint64_t tid;
-    set<SimpleLock*> locks;
     set<int> warning_ack_waiting;
     set<int> notify_ack_waiting;
     map<inodeno_t,map<client_t,Capability::Import> > peer_imported;
     list<Context*> waiting_for_finish;
+    Mutation *mut;
+    // for freeze tree deadlock detection
+    utime_t last_cum_auth_pins_change;
+    int last_cum_auth_pins;
+    int num_remote_waiters; // number of remote authpin waiters
+    export_state_t() : state(0), peer(0), tid(0), mut(NULL), 
+		       last_cum_auth_pins(0), num_remote_waiters(0) {}
   };
 
   map<CDir*, export_state_t>  export_state;
   
   list<pair<dirfrag_t,int> >  export_queue;
 
-  // for deadlock detection
-  struct freezing_state_t {
-    utime_t start_time;
-    int num_waiters;		// number of remote authpin waiters
-    freezing_state_t() : num_waiters(0) {}
-  };
-  map<CDir*,freezing_state_t >	export_freezing_state;
-  set<pair<utime_t,CDir*> >	export_freezing_dirs;
 
   // -- imports --
 public:
@@ -115,6 +115,7 @@ public:
   const static int IMPORT_ACKING        = 6; // logged EImportStart, sent ack, waiting for finish
   const static int IMPORT_FINISHING     = 7; // sent cap imports, waiting for finish
   const static int IMPORT_ABORTING      = 8; // notifying bystanders of an abort before unfreezing
+
   static const char *get_import_statename(int s) {
     switch (s) {
     case IMPORT_DISCOVERING: return "discovering";
@@ -139,13 +140,15 @@ protected:
     list<ScatterLock*> updated_scatterlocks;
     map<client_t,entity_inst_t> client_map;
     map<CInode*, map<client_t,Capability::Export> > peer_exports;
+    Mutation *mut;
+    import_state_t() : state(0), peer(0), tid(0), mut(NULL) {}
   };
 
   map<dirfrag_t, import_state_t>  import_state;
 
 public:
   // -- cons --
-  Migrator(MDS *m, MDCache *c) : mds(m), cache(c), last_export_tid(0) {}
+  Migrator(MDS *m, MDCache *c) : mds(m), cache(c) {}
 
   void dispatch(Message*);
 
@@ -211,8 +214,9 @@ public:
   }
 
   void export_freeze_inc_num_waiters(CDir *dir) {
-    assert(is_exporting(dir));
-    export_freezing_state[dir].num_waiters++;
+    map<CDir*, export_state_t>::iterator it = export_state.find(dir);
+    assert(it != export_state.end());
+    it->second.num_remote_waiters++;
   }
   void find_stale_export_freeze();
 
@@ -224,6 +228,7 @@ public:
   // -- import/export --
   // exporter
  public:
+  void dispatch_export_dir(MDRequest *mdr);
   void export_dir(CDir *dir, int dest);
   void export_empty_import(CDir *dir);
 
@@ -272,21 +277,14 @@ public:
   void export_sessions_flushed(CDir *dir, uint64_t tid);
   void export_go(CDir *dir);
   void export_go_synced(CDir *dir, uint64_t tid);
-  void export_try_cancel(CDir *dir);
+  void export_try_cancel(CDir *dir, bool notify_peer=true);
   void export_reverse(CDir *dir);
   void export_notify_abort(CDir *dir, set<CDir*>& bounds);
   void handle_export_ack(MExportDirAck *m);
   void export_logged_finish(CDir *dir);
   void handle_export_notify_ack(MExportDirNotifyAck *m);
-  void export_unlock(CDir *dir);
   void export_finish(CDir *dir);
 
-  void export_freeze_finish(CDir *dir) {
-    utime_t start = export_freezing_state[dir].start_time;
-    export_freezing_dirs.erase(make_pair(start, dir));
-    export_freezing_state.erase(dir);
-  }
-
   friend class C_MDC_ExportFreeze;
   friend class C_MDS_ExportFinishLogged;
   friend class C_M_ExportGo;
