@@ -19,8 +19,8 @@
 
 class PerfCounters;
 class LogEvent;
-class MDRequest;
-class Mutation;
+struct MDRequest;
+struct Mutation;
 class EMetaBlob;
 class EUpdate;
 class MMDSSlaveRequest;
@@ -78,7 +78,10 @@ public:
   version_t prepare_force_open_sessions(map<client_t,entity_inst_t> &cm,
 					map<client_t,uint64_t>& sseqmap);
   void finish_force_open_sessions(map<client_t,entity_inst_t> &cm,
-					map<client_t,uint64_t>& sseqmap);
+				  map<client_t,uint64_t>& sseqmap,
+				  bool dec_import=true);
+  void flush_client_sessions(set<client_t>& client_set, C_GatherBuilder& gather);
+  void finish_flush_session(Session *session, version_t seq);
   void terminate_sessions();
   void find_idle_sessions();
   void kill_session(Session *session);
@@ -139,14 +142,8 @@ public:
 
   // requests on existing inodes.
   void handle_client_getattr(MDRequest *mdr, bool is_lookup);
-  void handle_client_lookup_parent(MDRequest *mdr);
-  void handle_client_lookup_hash(MDRequest *mdr);
-  void _lookup_hash_2(MDRequest *mdr, int r);
-  void _lookup_hash_3(MDRequest *mdr, int r);
-  void handle_client_lookup_ino(MDRequest *mdr);
-  void _lookup_ino(MDRequest *mdr);
+  void handle_client_lookup_ino(MDRequest *mdr, bool want_parent, bool want_dentry);
   void _lookup_ino_2(MDRequest *mdr, int r);
-  void _lookup_ino_3(MDRequest *mdr, int r);
   void handle_client_readdir(MDRequest *mdr);
   void handle_client_file_setlock(MDRequest *mdr);
   void handle_client_file_readlock(MDRequest *mdr);
@@ -242,6 +239,8 @@ public:
   // slaving
   void handle_slave_rename_prep(MDRequest *mdr);
   void handle_slave_rename_prep_ack(MDRequest *mdr, MMDSSlaveRequest *m);
+  void handle_slave_rename_notify_ack(MDRequest *mdr, MMDSSlaveRequest *m);
+  void _slave_rename_sessions_flushed(MDRequest *mdr);
   void _logged_slave_rename(MDRequest *mdr, CDentry *srcdn, CDentry *destdn, CDentry *straydn);
   void _commit_slave_rename(MDRequest *mdr, int r, CDentry *srcdn, CDentry *destdn, CDentry *straydn);
   void do_rename_rollback(bufferlist &rbl, int master, MDRequest *mdr, bool finish_mdr=false);
