@@ -330,7 +330,8 @@ public:
     } else {
       state = s;
     }
-    take_waiting(SimpleLock::WAIT_ALL, waiters);
+    if (is_stable())
+      take_waiting(SimpleLock::WAIT_ALL, waiters);
   }
 
   bool is_stable() const {
@@ -368,13 +369,15 @@ public:
   }
 
   void init_gather() {
-    for (map<int,int>::const_iterator p = parent->replicas_begin(); 
+    for (map<int,unsigned>::const_iterator p = parent->replicas_begin();
 	 p != parent->replicas_end(); 
 	 ++p)
       more()->gather_set.insert(p->first);
   }
-  bool is_gathering() { return have_more() && !more()->gather_set.empty(); }
-  bool is_gathering(int i) {
+  bool is_gathering() const {
+    return have_more() && !more()->gather_set.empty();
+  }
+  bool is_gathering(int i) const {
     return have_more() && more()->gather_set.count(i);
   }
   void clear_gather() {
