@@ -20,30 +20,34 @@
 
 class MExportDirPrepAck : public Message {
   dirfrag_t dirfrag;
+  bool success;
 
  public:
   dirfrag_t get_dirfrag() { return dirfrag; }
   
   MExportDirPrepAck() {}
-  MExportDirPrepAck(dirfrag_t df, uint64_t tid) :
-    Message(MSG_MDS_EXPORTDIRPREPACK), dirfrag(df) {
+  MExportDirPrepAck(dirfrag_t df, bool s, uint64_t tid) :
+    Message(MSG_MDS_EXPORTDIRPREPACK), dirfrag(df), success(s) {
     set_tid(tid);
   }
 private:
   ~MExportDirPrepAck() {}
 
 public:  
+  bool is_success() { return success; }
   const char *get_type_name() const { return "ExPAck"; }
   void print(ostream& o) const {
-    o << "export_prep_ack(" << dirfrag << ")";
+    o << "export_prep_ack(" << dirfrag << (success ? " success)" : " fail)");
   }
 
   void decode_payload() {
     bufferlist::iterator p = payload.begin();
     ::decode(dirfrag, p);
+    ::decode(success, p);
   }
   void encode_payload(uint64_t features) {
     ::encode(dirfrag, payload);
+    ::encode(success, payload);
   }
 };
 
