@@ -12,7 +12,7 @@
  *
  */
 #include <memory>
-#include <tr1/memory>
+#include "include/memory.h"
 #include <errno.h>
 #include <map>
 #include <list>
