@@ -38,7 +38,7 @@ struct Subscription {
 };
 
 struct MonSession : public RefCountedObject {
-  Connection *con;
+  ConnectionRef con;
   entity_inst_t inst;
   utime_t until;
   utime_t time_established;
@@ -54,21 +54,17 @@ struct MonSession : public RefCountedObject {
 
   AuthServiceHandler *auth_handler;
 
-  Connection *proxy_con;
+  ConnectionRef proxy_con;
   uint64_t proxy_tid;
 
   MonSession(const entity_inst_t& i, Connection *c) :
-    con(c->get()), inst(i), closed(false), item(this),
+    con(c), inst(i), closed(false), item(this),
     auid(0),
     global_id(0), notified_global_id(0), auth_handler(NULL),
     proxy_con(NULL), proxy_tid(0) {
     time_established = ceph_clock_now(g_ceph_context);
   }
   ~MonSession() {
-    if (con)
-      con->put();
-    if (proxy_con)
-      proxy_con->put();
     //generic_dout(0) << "~MonSession " << this << dendl;
     // we should have been removed before we get destructed; see MonSessionMap::remove_session()
     assert(!item.is_on_list());
@@ -151,7 +147,8 @@ struct MonSessionMap {
     bool backward = true, forward = true;
     while (backward || forward) {
       if (backward) {
-        if (osdmap->is_up(b->first)) {
+        if (osdmap->is_up(b->first) &&
+	    osdmap->get_addr(b->first) == b->second->con->get_peer_addr()) {
           s = b->second;
           break;
         }
