@@ -6,7 +6,8 @@
 
 Messenger *Messenger::create(CephContext *cct,
 			     entity_name_t name,
+			     string lname,
 			     uint64_t nonce)
 {
-  return new SimpleMessenger(cct, name, nonce);
+  return new SimpleMessenger(cct, name, lname, nonce);
 }
