@@ -161,7 +161,9 @@ class DispatchQueue;
     DispatchQueue *in_q;
     list<Message*> sent;
     Cond cond;
-    bool keepalive;
+    bool send_keepalive;
+    bool send_keepalive_ack;
+    utime_t keepalive_ack_stamp;
     bool halt_delivery; //if a pipe's queue is destroyed, stop adding to it
     bool close_on_empty;
     
@@ -195,6 +197,7 @@ class DispatchQueue;
     int do_sendmsg(struct msghdr *msg, int len, bool more=false);
     int write_ack(uint64_t s);
     int write_keepalive();
+    int write_keepalive2(char tag, const utime_t &t);
 
     void fault(bool reader=false);
 
@@ -218,7 +221,7 @@ class DispatchQueue;
 
     __u32 get_out_seq() { return out_seq; }
 
-    bool is_queued() { return !out_q.empty() || keepalive; }
+    bool is_queued() { return !out_q.empty() || send_keepalive || send_keepalive_ack; }
 
     entity_addr_t& get_peer_addr() { return peer_addr; }
 
@@ -234,27 +237,21 @@ class DispatchQueue;
 
     void register_pipe();
     void unregister_pipe();
-    void join() {
-      if (writer_thread.is_started())
-        writer_thread.join();
-      if (reader_thread.is_started())
-        reader_thread.join();
-      if (delay_thread) {
-	delay_thread->stop();
-	delay_thread->join();
-      }
-    }
+    void join();
     void stop();
 
     void _send(Message *m) {
+      assert(pipe_lock.is_locked());
       out_q[m->get_priority()].push_back(m);
       cond.Signal();
     }
     void _send_keepalive() {
-      keepalive = true;
+      assert(pipe_lock.is_locked());
+      send_keepalive = true;
       cond.Signal();
     }
     Message *_get_next_outgoing() {
+      assert(pipe_lock.is_locked());
       Message *m = 0;
       while (!m && !out_q.empty()) {
         map<int, list<Message*> >::reverse_iterator p = out_q.rbegin();
