@@ -147,6 +147,7 @@ void SimpleMessenger::set_addr_unknowns(entity_addr_t &addr)
     int port = my_inst.addr.get_port();
     my_inst.addr.addr = addr.addr;
     my_inst.addr.set_port(port);
+    init_local_connection();
   }
 }
 
@@ -294,8 +295,10 @@ int SimpleMessenger::start()
   assert(!started);
   started = true;
 
-  if (!did_bind)
+  if (!did_bind) {
     my_inst.addr.nonce = nonce;
+    init_local_connection();
+  }
 
   lock.Unlock();
 
@@ -396,6 +399,19 @@ ConnectionRef SimpleMessenger::get_loopback_connection()
 void SimpleMessenger::submit_message(Message *m, Connection *con,
 				     const entity_addr_t& dest_addr, int dest_type, bool lazy)
 {
+
+  if (cct->_conf->ms_dump_on_send) {
+    m->encode(-1, true);
+    ldout(cct, 0) << "submit_message " << *m << "\n";
+    m->get_payload().hexdump(*_dout);
+    if (m->get_data().length() > 0) {
+      *_dout << " data:\n";
+      m->get_data().hexdump(*_dout);
+    }
+    *_dout << dendl;
+    m->clear_payload();
+  }
+
   // existing connection?
   if (con) {
     Pipe *pipe = NULL;
@@ -506,10 +522,12 @@ void SimpleMessenger::wait()
   }
   lock.Unlock();
 
-  ldout(cct,10) << "wait: waiting for dispatch queue" << dendl;
-  dispatch_queue.wait();
-  ldout(cct,10) << "wait: dispatch queue is stopped" << dendl;
-  
+  if(dispatch_queue.is_started()) {
+    ldout(cct,10) << "wait: waiting for dispatch queue" << dendl;
+    dispatch_queue.wait();
+    ldout(cct,10) << "wait: dispatch queue is stopped" << dendl;
+  }
+
   // done!  clean up.
   if (did_bind) {
     ldout(cct,20) << "wait: stopping accepter thread" << dendl;
