@@ -39,9 +39,9 @@ void entity_addr_t::generate_test_instances(list<entity_addr_t*>& o)
   b->set_nonce(5);
   b->set_family(AF_INET);
   b->set_in4_quad(0, 127);
-  b->set_in4_quad(0, 0);
-  b->set_in4_quad(0, 1);
-  b->set_in4_quad(0, 2);
+  b->set_in4_quad(1, 0);
+  b->set_in4_quad(2, 1);
+  b->set_in4_quad(3, 2);
   b->set_port(2);
   o.push_back(b);
 }
@@ -135,7 +135,7 @@ bool entity_addr_t::parse(const char *s, const char **end)
 ostream& operator<<(ostream& out, const sockaddr_storage &ss)
 {
   char buf[NI_MAXHOST] = { 0 };
-  char serv[20] = { 0 };
+  char serv[NI_MAXSERV] = { 0 };
   size_t hostlen;
 
   if (ss.ss_family == AF_INET)
