@@ -29,10 +29,14 @@ static ostream& _prefix(std::ostream *_dout, ReplicatedBackend *pgb) {
 }
 
 ReplicatedBackend::ReplicatedBackend(
-  PGBackend::Listener *pg, coll_t coll, OSDService *osd) :
-  PGBackend(pg), temp_created(false),
-  temp_coll(coll_t::make_temp_coll(pg->get_info().pgid)),
-  coll(coll), osd(osd), cct(osd->cct) {}
+  PGBackend::Listener *pg,
+  coll_t coll,
+  coll_t temp_coll,
+  ObjectStore *store,
+  CephContext *cct) :
+  PGBackend(pg, store,
+	    coll, temp_coll),
+  cct(cct) {}
 
 void ReplicatedBackend::run_recovery_op(
   PGBackend::RecoveryHandle *_h,
@@ -46,6 +50,7 @@ void ReplicatedBackend::run_recovery_op(
 
 void ReplicatedBackend::recover_object(
   const hobject_t &hoid,
+  eversion_t v,
   ObjectContextRef head,
   ObjectContextRef obc,
   RecoveryHandle *_h
@@ -57,6 +62,7 @@ void ReplicatedBackend::recover_object(
     assert(!obc);
     // pull
     prepare_pull(
+      v,
       hoid,
       head,
       h);
@@ -73,10 +79,10 @@ void ReplicatedBackend::recover_object(
 
 void ReplicatedBackend::check_recovery_sources(const OSDMapRef osdmap)
 {
-  for(map<int, set<hobject_t> >::iterator i = pull_from_peer.begin();
+  for(map<pg_shard_t, set<hobject_t> >::iterator i = pull_from_peer.begin();
       i != pull_from_peer.end();
       ) {
-    if (osdmap->is_down(i->first)) {
+    if (osdmap->is_down(i->first.osd)) {
       dout(10) << "check_recovery_sources resetting pulls from osd." << i->first
 	       << ", osdmap has it marked down" << dendl;
       for (set<hobject_t>::iterator j = i->second.begin();
@@ -186,19 +192,9 @@ void ReplicatedBackend::clear_state()
   pull_from_peer.clear();
 }
 
-void ReplicatedBackend::on_change(ObjectStore::Transaction *t)
+void ReplicatedBackend::_on_change(ObjectStore::Transaction *t)
 {
-  dout(10) << __func__ << dendl;
-  // clear temp
-  for (set<hobject_t>::iterator i = temp_contents.begin();
-       i != temp_contents.end();
-       ++i) {
-    dout(10) << __func__ << ": Removing oid "
-	     << *i << " from the temp collection" << dendl;
-    t->remove(get_temp_coll(t), *i);
-  }
-  temp_contents.clear();
-  for (map<tid_t, InProgressOp>::iterator i = in_progress_ops.begin();
+  for (map<ceph_tid_t, InProgressOp>::iterator i = in_progress_ops.begin();
        i != in_progress_ops.end();
        in_progress_ops.erase(i++)) {
     if (i->second.on_commit)
@@ -209,22 +205,12 @@ void ReplicatedBackend::on_change(ObjectStore::Transaction *t)
   clear_state();
 }
 
-coll_t ReplicatedBackend::get_temp_coll(ObjectStore::Transaction *t)
-{
-  if (temp_created)
-    return temp_coll;
-  if (!osd->store->collection_exists(temp_coll))
-      t->create_collection(temp_coll);
-  temp_created = true;
-  return temp_coll;
-}
-
 void ReplicatedBackend::on_flushed()
 {
   if (have_temp_coll() &&
-      !osd->store->collection_empty(get_temp_coll())) {
+      !store->collection_empty(get_temp_coll())) {
     vector<hobject_t> objects;
-    osd->store->collection_list(get_temp_coll(), objects);
+    store->collection_list(get_temp_coll(), objects);
     derr << __func__ << ": found objects in the temp collection: "
 	 << objects << ", crashing now"
 	 << dendl;
@@ -232,106 +218,13 @@ void ReplicatedBackend::on_flushed()
   }
 }
 
-
-int ReplicatedBackend::objects_list_partial(
-  const hobject_t &begin,
-  int min,
-  int max,
-  snapid_t seq,
-  vector<hobject_t> *ls,
-  hobject_t *next)
-{
-  assert(ls);
-  ghobject_t _next(begin);
-  ls->reserve(max);
-  int r = 0;
-  while (!_next.is_max() && ls->size() < (unsigned)min) {
-    vector<ghobject_t> objects;
-    int r = osd->store->collection_list_partial(
-      coll,
-      _next,
-      min - ls->size(),
-      max - ls->size(),
-      seq,
-      &objects,
-      &_next);
-    if (r != 0)
-      break;
-    for (vector<ghobject_t>::iterator i = objects.begin();
-	 i != objects.end();
-	 ++i) {
-      assert(i->is_no_shard());
-      if (i->is_no_gen()) {
-	ls->push_back(i->hobj);
-      }
-    }
-  }
-  if (r == 0)
-    *next = _next.hobj;
-  return r;
-}
-
-int ReplicatedBackend::objects_list_range(
-  const hobject_t &start,
-  const hobject_t &end,
-  snapid_t seq,
-  vector<hobject_t> *ls)
-{
-  assert(ls);
-  vector<ghobject_t> objects;
-  int r = osd->store->collection_list_range(
-    coll,
-    start,
-    end,
-    seq,
-    &objects);
-  ls->reserve(objects.size());
-  for (vector<ghobject_t>::iterator i = objects.begin();
-       i != objects.end();
-       ++i) {
-    assert(i->is_no_shard());
-    if (i->is_no_gen()) {
-      ls->push_back(i->hobj);
-    }
-  }
-  return r;
-}
-
-int ReplicatedBackend::objects_get_attr(
-  const hobject_t &hoid,
-  const string &attr,
-  bufferlist *out)
-{
-  bufferptr bp;
-  int r = osd->store->getattr(
-    coll,
-    hoid,
-    attr.c_str(),
-    bp);
-  if (r >= 0 && out) {
-    out->clear();
-    out->push_back(bp);
-  }
-  return r;
-}
-
-int ReplicatedBackend::objects_get_attrs(
-  const hobject_t &hoid,
-  map<string, bufferlist> *out)
-{
-  return osd->store->getattrs(
-    coll,
-    hoid,
-    *out);
-}
-
 int ReplicatedBackend::objects_read_sync(
   const hobject_t &hoid,
   uint64_t off,
   uint64_t len,
   bufferlist *bl)
 {
-  return osd->store->read(coll, hoid, off, len, *bl);
+  return store->read(coll, hoid, off, len, *bl);
 }
 
 struct AsyncReadCallback : public GenContext<ThreadPool::TPHandle&> {
@@ -358,17 +251,17 @@ void ReplicatedBackend::objects_read_async(
 	   to_read.begin();
        i != to_read.end() && r >= 0;
        ++i) {
-    int _r = osd->store->read(coll, hoid, i->first.first,
-			      i->first.second, *(i->second.first));
+    int _r = store->read(coll, hoid, i->first.first,
+			 i->first.second, *(i->second.first));
     if (i->second.second) {
-      osd->gen_wq.queue(
+      get_parent()->schedule_work(
 	get_parent()->bless_gencontext(
 	  new AsyncReadCallback(_r, i->second.second)));
     }
     if (_r < 0)
       r = _r;
   }
-  osd->gen_wq.queue(
+  get_parent()->schedule_work(
     get_parent()->bless_gencontext(
       new AsyncReadCallback(r, on_complete)));
 }
@@ -436,7 +329,7 @@ public:
     version_t former_version) {
     t->collection_move_rename(
       coll, hoid, coll,
-      ghobject_t(hoid, former_version, 0));
+      ghobject_t(hoid, former_version, ghobject_t::NO_SHARD));
   }
   void setattrs(
     const hobject_t &hoid,
@@ -528,10 +421,20 @@ public:
     t->zero(get_coll(hoid), hoid, off, len);
   }
 
+  void set_alloc_hint(
+    const hobject_t &hoid,
+    uint64_t expected_object_size,
+    uint64_t expected_write_size
+    ) {
+    t->set_alloc_hint(get_coll(hoid), hoid, expected_object_size,
+                      expected_write_size);
+  }
+
   void append(
     PGTransaction *_to_append
     ) {
     RPGTransaction *to_append = dynamic_cast<RPGTransaction*>(_to_append);
+    assert(to_append);
     t->append(*(to_append->t));
     for (set<hobject_t>::iterator i = to_append->temp_added.begin();
 	 i != to_append->temp_added.end();
@@ -594,11 +497,12 @@ void ReplicatedBackend::submit_transaction(
   Context *on_local_applied_sync,
   Context *on_all_acked,
   Context *on_all_commit,
-  tid_t tid,
+  ceph_tid_t tid,
   osd_reqid_t reqid,
   OpRequestRef orig_op)
 {
   RPGTransaction *t = dynamic_cast<RPGTransaction*>(_t);
+  assert(t);
   ObjectStore::Transaction *op_t = t->get_transaction();
 
   assert(t->get_temp_added().size() <= 1);
@@ -614,6 +518,14 @@ void ReplicatedBackend::submit_transaction(
       )
     ).first->second;
 
+  op.waiting_for_applied.insert(
+    parent->get_actingbackfill_shards().begin(),
+    parent->get_actingbackfill_shards().end());
+  op.waiting_for_commit.insert(
+    parent->get_actingbackfill_shards().begin(),
+    parent->get_actingbackfill_shards().end());
+
+
   issue_op(
     soid,
     at_version,
@@ -627,20 +539,13 @@ void ReplicatedBackend::submit_transaction(
     &op,
     op_t);
 
-  // add myself to gather set
-  op.waiting_for_applied.insert(osd->whoami);
-  op.waiting_for_commit.insert(osd->whoami);
-
   ObjectStore::Transaction local_t;
   if (t->get_temp_added().size()) {
     get_temp_coll(&local_t);
-    temp_contents.insert(t->get_temp_added().begin(), t->get_temp_added().end());
-  }
-  for (set<hobject_t>::const_iterator i = t->get_temp_cleared().begin();
-       i != t->get_temp_cleared().end();
-       ++i) {
-    temp_contents.erase(*i);
+    add_temp_objs(t->get_temp_added());
   }
+  clear_temp_objs(t->get_temp_cleared());
+
   parent->log_operation(log_entries, trim_to, true, &local_t);
   local_t.append(*op_t);
   local_t.swap(*op_t);
@@ -666,7 +571,7 @@ void ReplicatedBackend::op_applied(
   if (op->op)
     op->op->mark_event("op_applied");
 
-  op->waiting_for_applied.erase(osd->whoami);
+  op->waiting_for_applied.erase(get_parent()->whoami_shard());
   parent->op_applied(op->v);
 
   if (op->waiting_for_applied.empty()) {
@@ -686,7 +591,7 @@ void ReplicatedBackend::op_commit(
   if (op->op)
     op->op->mark_event("op_commit");
 
-  op->waiting_for_commit.erase(osd->whoami);
+  op->waiting_for_commit.erase(get_parent()->whoami_shard());
 
   if (op->waiting_for_commit.empty()) {
     op->on_commit->complete(0);
@@ -706,11 +611,11 @@ void ReplicatedBackend::sub_op_modify_reply(OpRequestRef op)
   op->mark_started();
 
   // must be replication.
-  tid_t rep_tid = r->get_tid();
-  int fromosd = r->get_source().num();
+  ceph_tid_t rep_tid = r->get_tid();
+  pg_shard_t from = r->from;
 
   if (in_progress_ops.count(rep_tid)) {
-    map<tid_t, InProgressOp>::iterator iter =
+    map<ceph_tid_t, InProgressOp>::iterator iter =
       in_progress_ops.find(rep_tid);
     InProgressOp &ip_op = iter->second;
     MOSDOp *m = NULL;
@@ -719,31 +624,31 @@ void ReplicatedBackend::sub_op_modify_reply(OpRequestRef op)
 
     if (m)
       dout(7) << __func__ << ": tid " << ip_op.tid << " op " //<< *m
-	      << " ack_type " << r->ack_type
-	      << " from osd." << fromosd
+	      << " ack_type " << (int)r->ack_type
+	      << " from " << from
 	      << dendl;
     else
       dout(7) << __func__ << ": tid " << ip_op.tid << " (no op) "
-	      << " ack_type " << r->ack_type
-	      << " from osd." << fromosd
+	      << " ack_type " << (int)r->ack_type
+	      << " from " << from
 	      << dendl;
 
     // oh, good.
 
     if (r->ack_type & CEPH_OSD_FLAG_ONDISK) {
-      assert(ip_op.waiting_for_commit.count(fromosd));
-      ip_op.waiting_for_commit.erase(fromosd);
+      assert(ip_op.waiting_for_commit.count(from));
+      ip_op.waiting_for_commit.erase(from);
       if (ip_op.op)
 	ip_op.op->mark_event("sub_op_commit_rec");
     } else {
-      assert(ip_op.waiting_for_applied.count(fromosd));
+      assert(ip_op.waiting_for_applied.count(from));
       if (ip_op.op)
 	ip_op.op->mark_event("sub_op_applied_rec");
     }
-    ip_op.waiting_for_applied.erase(fromosd);
+    ip_op.waiting_for_applied.erase(from);
 
     parent->update_peer_last_complete_ondisk(
-      fromosd,
+      from,
       r->get_last_complete_ondisk());
 
     if (ip_op.waiting_for_applied.empty() &&
@@ -763,317 +668,80 @@ void ReplicatedBackend::sub_op_modify_reply(OpRequestRef op)
   }
 }
 
-/*
- * pg lock may or may not be held
- */
-void ReplicatedBackend::be_scan_list(
-  ScrubMap &map, const vector<hobject_t> &ls, bool deep,
-  ThreadPool::TPHandle &handle)
-{
-  dout(10) << "_scan_list scanning " << ls.size() << " objects"
-           << (deep ? " deeply" : "") << dendl;
-  int i = 0;
-  for (vector<hobject_t>::const_iterator p = ls.begin();
-       p != ls.end();
-       ++p, i++) {
+void ReplicatedBackend::be_deep_scrub(
+  const hobject_t &poid,
+  ScrubMap::object &o,
+  ThreadPool::TPHandle &handle) {
+  bufferhash h, oh;
+  bufferlist bl, hdrbl;
+  int r;
+  __u64 pos = 0;
+  while ( (r = store->read(
+	     coll,
+	     ghobject_t(
+	       poid, ghobject_t::NO_GEN, get_parent()->whoami_shard().shard),
+	     pos,
+	     cct->_conf->osd_deep_scrub_stride, bl,
+	     true)) > 0) {
     handle.reset_tp_timeout();
-    hobject_t poid = *p;
+    h << bl;
+    pos += bl.length();
+    bl.clear();
+  }
+  if (r == -EIO) {
+    dout(25) << "_scan_list  " << poid << " got "
+	     << r << " on read, read_error" << dendl;
+    o.read_error = true;
+  }
+  o.digest = h.digest();
+  o.digest_present = true;
 
-    struct stat st;
-    int r = osd->store->stat(coll, poid, &st, true);
-    if (r == 0) {
-      ScrubMap::object &o = map.objects[poid];
-      o.size = st.st_size;
-      assert(!o.negative);
-      osd->store->getattrs(coll, poid, o.attrs);
-
-      // calculate the CRC32 on deep scrubs
-      if (deep) {
-        bufferhash h, oh;
-        bufferlist bl, hdrbl;
-        int r;
-        __u64 pos = 0;
-        while ( (r = osd->store->read(coll, poid, pos,
-                                       cct->_conf->osd_deep_scrub_stride, bl,
-		                      true)) > 0) {
-	  handle.reset_tp_timeout();
-          h << bl;
-          pos += bl.length();
-          bl.clear();
-        }
-	if (r == -EIO) {
-	  dout(25) << "_scan_list  " << poid << " got "
-		   << r << " on read, read_error" << dendl;
-	  o.read_error = true;
-	}
-        o.digest = h.digest();
-        o.digest_present = true;
-
-        bl.clear();
-        r = osd->store->omap_get_header(coll, poid, &hdrbl, true);
-        if (r == 0) {
-          dout(25) << "CRC header " << string(hdrbl.c_str(), hdrbl.length())
+  bl.clear();
+  r = store->omap_get_header(
+    coll,
+    ghobject_t(
+      poid, ghobject_t::NO_GEN, get_parent()->whoami_shard().shard),
+    &hdrbl, true);
+  if (r == 0) {
+    dout(25) << "CRC header " << string(hdrbl.c_str(), hdrbl.length())
              << dendl;
-          ::encode(hdrbl, bl);
-          oh << bl;
-          bl.clear();
-        } else if (r == -EIO) {
-	  dout(25) << "_scan_list  " << poid << " got "
-		   << r << " on omap header read, read_error" << dendl;
-	  o.read_error = true;
-	}
-
-        ObjectMap::ObjectMapIterator iter = osd->store->get_omap_iterator(
-          coll, poid);
-        assert(iter);
-	uint64_t keys_scanned = 0;
-        for (iter->seek_to_first(); iter->valid() ; iter->next()) {
-	  if (cct->_conf->osd_scan_list_ping_tp_interval &&
-	      (keys_scanned % cct->_conf->osd_scan_list_ping_tp_interval == 0)) {
-	    handle.reset_tp_timeout();
-	  }
-	  ++keys_scanned;
-
-          dout(25) << "CRC key " << iter->key() << " value "
-            << string(iter->value().c_str(), iter->value().length()) << dendl;
-
-          ::encode(iter->key(), bl);
-          ::encode(iter->value(), bl);
-          oh << bl;
-          bl.clear();
-        }
-	if (iter->status() == -EIO) {
-	  dout(25) << "_scan_list  " << poid << " got "
-		   << r << " on omap scan, read_error" << dendl;
-	  o.read_error = true;
-	  break;
-	}
-
-        //Store final calculated CRC32 of omap header & key/values
-        o.omap_digest = oh.digest();
-        o.omap_digest_present = true;
-      }
-
-      dout(25) << "_scan_list  " << poid << dendl;
-    } else if (r == -ENOENT) {
-      dout(25) << "_scan_list  " << poid << " got " << r << ", skipping" << dendl;
-    } else if (r == -EIO) {
-      dout(25) << "_scan_list  " << poid << " got " << r << ", read_error" << dendl;
-      ScrubMap::object &o = map.objects[poid];
-      o.read_error = true;
-    } else {
-      derr << "_scan_list got: " << cpp_strerror(r) << dendl;
-      assert(0);
-    }
-  }
-}
-
-enum scrub_error_type ReplicatedBackend::be_compare_scrub_objects(
-				const ScrubMap::object &auth,
-				const ScrubMap::object &candidate,
-				ostream &errorstream)
-{
-  enum scrub_error_type error = CLEAN;
-  if (candidate.read_error) {
-    // This can occur on stat() of a shallow scrub, but in that case size will
-    // be invalid, and this will be over-ridden below.
-    error = DEEP_ERROR;
-    errorstream << "candidate had a read error";
-  }
-  if (auth.digest_present && candidate.digest_present) {
-    if (auth.digest != candidate.digest) {
-      if (error != CLEAN)
-        errorstream << ", ";
-      error = DEEP_ERROR;
-
-      errorstream << "digest " << candidate.digest
-                  << " != known digest " << auth.digest;
-    }
-  }
-  if (auth.omap_digest_present && candidate.omap_digest_present) {
-    if (auth.omap_digest != candidate.omap_digest) {
-      if (error != CLEAN)
-        errorstream << ", ";
-      error = DEEP_ERROR;
-
-      errorstream << "omap_digest " << candidate.omap_digest
-                  << " != known omap_digest " << auth.omap_digest;
-    }
-  }
-  // Shallow error takes precendence because this will be seen by
-  // both types of scrubs.
-  if (auth.size != candidate.size) {
-    if (error != CLEAN)
-      errorstream << ", ";
-    error = SHALLOW_ERROR;
-    errorstream << "size " << candidate.size
-		<< " != known size " << auth.size;
-  }
-  for (map<string,bufferptr>::const_iterator i = auth.attrs.begin();
-       i != auth.attrs.end();
-       ++i) {
-    if (!candidate.attrs.count(i->first)) {
-      if (error != CLEAN)
-        errorstream << ", ";
-      error = SHALLOW_ERROR;
-      errorstream << "missing attr " << i->first;
-    } else if (candidate.attrs.find(i->first)->second.cmp(i->second)) {
-      if (error != CLEAN)
-        errorstream << ", ";
-      error = SHALLOW_ERROR;
-      errorstream << "attr value mismatch " << i->first;
-    }
-  }
-  for (map<string,bufferptr>::const_iterator i = candidate.attrs.begin();
-       i != candidate.attrs.end();
-       ++i) {
-    if (!auth.attrs.count(i->first)) {
-      if (error != CLEAN)
-        errorstream << ", ";
-      error = SHALLOW_ERROR;
-      errorstream << "extra attr " << i->first;
-    }
-  }
-  return error;
-}
-
-map<int, ScrubMap *>::const_iterator ReplicatedBackend::be_select_auth_object(
-  const hobject_t &obj,
-  const map<int,ScrubMap*> &maps)
-{
-  map<int, ScrubMap *>::const_iterator auth = maps.end();
-  for (map<int, ScrubMap *>::const_iterator j = maps.begin();
-       j != maps.end();
-       ++j) {
-    map<hobject_t, ScrubMap::object>::iterator i =
-      j->second->objects.find(obj);
-    if (i == j->second->objects.end()) {
-      continue;
-    }
-    if (auth == maps.end()) {
-      // Something is better than nothing
-      // TODO: something is NOT better than nothing, do something like
-      // unfound_lost if no valid copies can be found, or just mark unfound
-      auth = j;
-      dout(10) << __func__ << ": selecting osd " << j->first
-	       << " for obj " << obj
-	       << ", auth == maps.end()"
-	       << dendl;
-      continue;
-    }
-    if (i->second.read_error) {
-      // scrub encountered read error, probably corrupt
-      dout(10) << __func__ << ": rejecting osd " << j->first
-	       << " for obj " << obj
-	       << ", read_error"
-	       << dendl;
-      continue;
-    }
-    map<string, bufferptr>::iterator k = i->second.attrs.find(OI_ATTR);
-    if (k == i->second.attrs.end()) {
-      // no object info on object, probably corrupt
-      dout(10) << __func__ << ": rejecting osd " << j->first
-	       << " for obj " << obj
-	       << ", no oi attr"
-	       << dendl;
-      continue;
-    }
-    bufferlist bl;
-    bl.push_back(k->second);
-    object_info_t oi;
-    try {
-      bufferlist::iterator bliter = bl.begin();
-      ::decode(oi, bliter);
-    } catch (...) {
-      dout(10) << __func__ << ": rejecting osd " << j->first
-	       << " for obj " << obj
-	       << ", corrupt oi attr"
-	       << dendl;
-      // invalid object info, probably corrupt
-      continue;
-    }
-    if (oi.size != i->second.size) {
-      // invalid size, probably corrupt
-      dout(10) << __func__ << ": rejecting osd " << j->first
-	       << " for obj " << obj
-	       << ", size mismatch"
-	       << dendl;
-      // invalid object info, probably corrupt
-      continue;
-    }
-    dout(10) << __func__ << ": selecting osd " << j->first
-	     << " for obj " << obj
-	     << dendl;
-    auth = j;
-  }
-  return auth;
-}
-
-void ReplicatedBackend::be_compare_scrubmaps(const map<int,ScrubMap*> &maps,
-			    map<hobject_t, set<int> > &missing,
-			    map<hobject_t, set<int> > &inconsistent,
-			    map<hobject_t, int> &authoritative,
-			    map<hobject_t, set<int> > &invalid_snapcolls,
-			    int &shallow_errors,
-			    int &deep_errors,
-			    const pg_t pgid,
-			    const vector<int> &acting,
-			    ostream &errorstream)
-{
-  map<hobject_t,ScrubMap::object>::const_iterator i;
-  map<int, ScrubMap *>::const_iterator j;
-  set<hobject_t> master_set;
-
-  // Construct master set
-  for (j = maps.begin(); j != maps.end(); ++j) {
-    for (i = j->second->objects.begin(); i != j->second->objects.end(); ++i) {
-      master_set.insert(i->first);
-    }
-  }
-
-  // Check maps against master set and each other
-  for (set<hobject_t>::const_iterator k = master_set.begin();
-       k != master_set.end();
-       ++k) {
-    map<int, ScrubMap *>::const_iterator auth = be_select_auth_object(*k, maps);
-    assert(auth != maps.end());
-    set<int> cur_missing;
-    set<int> cur_inconsistent;
-    for (j = maps.begin(); j != maps.end(); ++j) {
-      if (j == auth)
-	continue;
-      if (j->second->objects.count(*k)) {
-	// Compare
-	stringstream ss;
-	enum scrub_error_type error = be_compare_scrub_objects(auth->second->objects[*k],
-	    j->second->objects[*k],
-	    ss);
-        if (error != CLEAN) {
-	  cur_inconsistent.insert(j->first);
-          if (error == SHALLOW_ERROR)
-	    ++shallow_errors;
-          else
-	    ++deep_errors;
-	  errorstream << pgid << " osd." << acting[j->first]
-		      << ": soid " << *k << " " << ss.str() << std::endl;
-	}
-      } else {
-	cur_missing.insert(j->first);
-	++shallow_errors;
-	errorstream << pgid
-		    << " osd." << acting[j->first]
-		    << " missing " << *k << std::endl;
-      }
-    }
-    assert(auth != maps.end());
-    if (!cur_missing.empty()) {
-      missing[*k] = cur_missing;
-    }
-    if (!cur_inconsistent.empty()) {
-      inconsistent[*k] = cur_inconsistent;
-    }
-    if (!cur_inconsistent.empty() || !cur_missing.empty()) {
-      authoritative[*k] = auth->first;
-    }
+    ::encode(hdrbl, bl);
+    oh << bl;
+    bl.clear();
+  } else if (r == -EIO) {
+    dout(25) << "_scan_list  " << poid << " got "
+	     << r << " on omap header read, read_error" << dendl;
+    o.read_error = true;
   }
+
+  ObjectMap::ObjectMapIterator iter = store->get_omap_iterator(
+    coll,
+    ghobject_t(
+      poid, ghobject_t::NO_GEN, get_parent()->whoami_shard().shard));
+  assert(iter);
+  uint64_t keys_scanned = 0;
+  for (iter->seek_to_first(); iter->valid() ; iter->next()) {
+    if (cct->_conf->osd_scan_list_ping_tp_interval &&
+	(keys_scanned % cct->_conf->osd_scan_list_ping_tp_interval == 0)) {
+      handle.reset_tp_timeout();
+    }
+    ++keys_scanned;
+
+    dout(25) << "CRC key " << iter->key() << " value "
+	     << string(iter->value().c_str(), iter->value().length()) << dendl;
+
+    ::encode(iter->key(), bl);
+    ::encode(iter->value(), bl);
+    oh << bl;
+    bl.clear();
+  }
+  if (iter->status() == -EIO) {
+    dout(25) << "_scan_list  " << poid << " got "
+	     << r << " on omap scan, read_error" << dendl;
+    o.read_error = true;
+  }
+
+  //Store final calculated CRC32 of omap header & key/values
+  o.omap_digest = oh.digest();
+  o.omap_digest_present = true;
 }
