@@ -334,3 +334,112 @@ TEST(CephArgParse, WithInt) {
   ASSERT_EQ(foo, 40);
   ASSERT_EQ(bar, -1);
 }
+
+TEST(CephArgParse, env_to_vec) {
+  {
+    std::vector<const char*> args;
+    unsetenv("CEPH_ARGS");
+    unsetenv("WHATEVER");
+    env_to_vec(args);
+    EXPECT_EQ(0u, args.size());
+    env_to_vec(args, "WHATEVER");
+    EXPECT_EQ(0u, args.size());
+    args.push_back("a");
+    setenv("CEPH_ARGS", "b c", 0);
+    env_to_vec(args);
+    EXPECT_EQ(3u, args.size());
+    EXPECT_EQ(string("b"), args[1]);
+    EXPECT_EQ(string("c"), args[2]);
+    setenv("WHATEVER", "d e", 0);
+    env_to_vec(args, "WHATEVER");
+    EXPECT_EQ(5u, args.size());
+    EXPECT_EQ(string("d"), args[3]);
+    EXPECT_EQ(string("e"), args[4]);
+  }
+  {
+    std::vector<const char*> args;
+    unsetenv("CEPH_ARGS");
+    args.push_back("a");
+    args.push_back("--");
+    args.push_back("c");
+    setenv("CEPH_ARGS", "b -- d", 0);
+    env_to_vec(args);
+    EXPECT_EQ(5u, args.size());
+    EXPECT_EQ(string("a"), args[0]);
+    EXPECT_EQ(string("b"), args[1]);
+    EXPECT_EQ(string("--"), args[2]);
+    EXPECT_EQ(string("c"), args[3]);
+    EXPECT_EQ(string("d"), args[4]);
+  }
+  {
+    std::vector<const char*> args;
+    unsetenv("CEPH_ARGS");
+    args.push_back("a");
+    args.push_back("--");
+    setenv("CEPH_ARGS", "b -- c", 0);
+    env_to_vec(args);
+    EXPECT_EQ(4u, args.size());
+    EXPECT_EQ(string("a"), args[0]);
+    EXPECT_EQ(string("b"), args[1]);
+    EXPECT_EQ(string("--"), args[2]);
+    EXPECT_EQ(string("c"), args[3]);
+  }
+  {
+    std::vector<const char*> args;
+    unsetenv("CEPH_ARGS");
+    args.push_back("--");
+    args.push_back("c");
+    setenv("CEPH_ARGS", "b -- d", 0);
+    env_to_vec(args);
+    EXPECT_EQ(4u, args.size());
+    EXPECT_EQ(string("b"), args[0]);
+    EXPECT_EQ(string("--"), args[1]);
+    EXPECT_EQ(string("c"), args[2]);
+    EXPECT_EQ(string("d"), args[3]);
+  }
+  {
+    std::vector<const char*> args;
+    unsetenv("CEPH_ARGS");
+    args.push_back("b");
+    setenv("CEPH_ARGS", "c -- d", 0);
+    env_to_vec(args);
+    EXPECT_EQ(4u, args.size());
+    EXPECT_EQ(string("b"), args[0]);
+    EXPECT_EQ(string("c"), args[1]);
+    EXPECT_EQ(string("--"), args[2]);
+    EXPECT_EQ(string("d"), args[3]);
+  }
+  {
+    std::vector<const char*> args;
+    unsetenv("CEPH_ARGS");
+    args.push_back("a");
+    args.push_back("--");
+    args.push_back("c");
+    setenv("CEPH_ARGS", "-- d", 0);
+    env_to_vec(args);
+    EXPECT_EQ(4u, args.size());
+    EXPECT_EQ(string("a"), args[0]);
+    EXPECT_EQ(string("--"), args[1]);
+    EXPECT_EQ(string("c"), args[2]);
+    EXPECT_EQ(string("d"), args[3]);
+  }
+  {
+    std::vector<const char*> args;
+    unsetenv("CEPH_ARGS");
+    args.push_back("a");
+    args.push_back("--");
+    args.push_back("c");
+    setenv("CEPH_ARGS", "d", 0);
+    env_to_vec(args);
+    EXPECT_EQ(4u, args.size());
+    EXPECT_EQ(string("a"), args[0]);
+    EXPECT_EQ(string("d"), args[1]);
+    EXPECT_EQ(string("--"), args[2]);
+    EXPECT_EQ(string("c"), args[3]);
+  }
+}
+/*
+ * Local Variables:
+ * compile-command: "cd .. ; make unittest_ceph_argparse && ./unittest_ceph_argparse"
+ * End:
+ */
