@@ -3,9 +3,8 @@
 
 #include "common/snap_types.h"
 #include "include/encoding.h"
-#include "include/rados.h"
-#include "include/rados/librados.h"
 #include "include/types.h"
+#include "include/rados/librados.h"
 #include "cls/rbd/cls_rbd.h"
 #include "cls/rbd/cls_rbd_client.h"
 
@@ -310,7 +309,7 @@ TEST(cls_rbd, create)
   ASSERT_EQ(0, rados.ioctx_create(pool_name.c_str(), ioctx));
 
   string oid = "testobj";
-  uint64_t size = 20 << 30;
+  uint64_t size = 20ULL << 30;
   uint64_t features = 0;
   uint8_t order = 22;
   string object_prefix = "foo";
@@ -477,37 +476,37 @@ TEST(cls_rbd, protection_status)
   ASSERT_EQ(0, snapshot_add(&ioctx, "foo", 10, "snap1"));
   ASSERT_EQ(0, get_protection_status(&ioctx, "foo",
 				     10, &status));
-  ASSERT_EQ(RBD_PROTECTION_STATUS_UNPROTECTED, status);
+  ASSERT_EQ(+RBD_PROTECTION_STATUS_UNPROTECTED, status);
 
   ASSERT_EQ(0, set_protection_status(&ioctx, "foo",
 				     10, RBD_PROTECTION_STATUS_PROTECTED));
   ASSERT_EQ(0, get_protection_status(&ioctx, "foo",
 				     10, &status));
-  ASSERT_EQ(RBD_PROTECTION_STATUS_PROTECTED, status);
+  ASSERT_EQ(+RBD_PROTECTION_STATUS_PROTECTED, status);
   ASSERT_EQ(-EBUSY, snapshot_remove(&ioctx, "foo", 10));
 
   ASSERT_EQ(0, set_protection_status(&ioctx, "foo",
 				     10, RBD_PROTECTION_STATUS_UNPROTECTING));
   ASSERT_EQ(0, get_protection_status(&ioctx, "foo",
 				     10, &status));
-  ASSERT_EQ(RBD_PROTECTION_STATUS_UNPROTECTING, status);
+  ASSERT_EQ(+RBD_PROTECTION_STATUS_UNPROTECTING, status);
   ASSERT_EQ(-EBUSY, snapshot_remove(&ioctx, "foo", 10));
 
   ASSERT_EQ(-EINVAL, set_protection_status(&ioctx, "foo",
 					   10, RBD_PROTECTION_STATUS_LAST));
   ASSERT_EQ(0, get_protection_status(&ioctx, "foo",
 				     10, &status));
-  ASSERT_EQ(RBD_PROTECTION_STATUS_UNPROTECTING, status);
+  ASSERT_EQ(+RBD_PROTECTION_STATUS_UNPROTECTING, status);
 
   ASSERT_EQ(0, snapshot_add(&ioctx, "foo", 20, "snap2"));
   ASSERT_EQ(0, get_protection_status(&ioctx, "foo",
 				     20, &status));
-  ASSERT_EQ(RBD_PROTECTION_STATUS_UNPROTECTED, status);
+  ASSERT_EQ(+RBD_PROTECTION_STATUS_UNPROTECTED, status);
   ASSERT_EQ(0, set_protection_status(&ioctx, "foo",
 				     10, RBD_PROTECTION_STATUS_UNPROTECTED));
   ASSERT_EQ(0, get_protection_status(&ioctx, "foo",
 				     10, &status));
-  ASSERT_EQ(RBD_PROTECTION_STATUS_UNPROTECTED, status);
+  ASSERT_EQ(+RBD_PROTECTION_STATUS_UNPROTECTED, status);
 
   ASSERT_EQ(0, snapshot_remove(&ioctx, "foo", 10));
   ASSERT_EQ(0, snapshot_remove(&ioctx, "foo", 20));
