@@ -14,13 +14,13 @@
 using std::ostringstream;
 using namespace librados;
 using std::pair;
-using std::make_pair;
 
 class AioTestData
 {
 public:
   AioTestData()
     : m_cluster(NULL),
+      m_ioctx(NULL),
       m_init(false),
       m_complete(false),
       m_safe(false)
@@ -177,16 +177,17 @@ TEST(LibRadosAio, SimpleWrite) {
 }
 
 TEST(LibRadosAio, SimpleWritePP) {
+  char buf[128];
+  memset(buf, 0xcc, sizeof(buf));
+  bufferlist bl1;
+  bl1.append(buf, sizeof(buf));
+  {
   AioTestDataPP test_data;
   ASSERT_EQ("", test_data.init());
   AioCompletion *my_completion = test_data.m_cluster.aio_create_completion(
 	  (void*)&test_data, set_completion_complete, set_completion_safe);
   AioCompletion *my_completion_null = NULL;
   ASSERT_NE(my_completion, my_completion_null);
-  char buf[128];
-  memset(buf, 0xcc, sizeof(buf));
-  bufferlist bl1;
-  bl1.append(buf, sizeof(buf));
   ASSERT_EQ(0, test_data.m_ioctx.aio_write("foo",
 			       my_completion, bl1, sizeof(buf), 0));
   {
@@ -195,10 +196,13 @@ TEST(LibRadosAio, SimpleWritePP) {
     sem_wait(&test_data.m_sem);
   }
   delete my_completion;
+  }
 
+  {
+  AioTestDataPP test_data;
   ASSERT_EQ("", test_data.init());
   test_data.m_ioctx.set_namespace("nspace");
-  my_completion = test_data.m_cluster.aio_create_completion(
+  AioCompletion *my_completion = test_data.m_cluster.aio_create_completion(
 	  (void*)&test_data, set_completion_complete, set_completion_safe);
   ASSERT_EQ(0, test_data.m_ioctx.aio_write("foo",
 			       my_completion, bl1, sizeof(buf), 0));
@@ -208,6 +212,7 @@ TEST(LibRadosAio, SimpleWritePP) {
     sem_wait(&test_data.m_sem);
   }
   delete my_completion;
+  }
 }
 
 TEST(LibRadosAio, WaitForSafe) {
@@ -1157,7 +1162,6 @@ TEST(LibRadosAio, OmapPP) {
   {
     boost::scoped_ptr<AioCompletion> my_completion(cluster.aio_create_completion(0, 0, 0));
     ObjectWriteOperation op;
-    string val = "bar";
     to_set["foo"] = header_to_set;
     to_set["foo2"] = header_to_set;
     to_set["qfoo3"] = header_to_set;
@@ -1188,7 +1192,7 @@ TEST(LibRadosAio, OmapPP) {
       TestAlarm alarm;
       ASSERT_EQ(0, my_completion->wait_for_complete());
     }
-    ASSERT_EQ(0, r);
+    ASSERT_EQ(-ECANCELED, r);
   }
 
   {
@@ -1287,4 +1291,5 @@ TEST(LibRadosAio, OmapPP) {
   }
 
   ioctx.remove("test_obj");
+  destroy_one_pool_pp(pool_name, cluster);
 }
