@@ -18,6 +18,10 @@
 #include "include/rbd/librbd.h"
 #include "include/rbd/librbd.hpp"
 
+#include "global/global_context.h"
+#include "global/global_init.h"
+#include "common/ceph_argparse.h"
+
 #include "gtest/gtest.h"
 
 #include <errno.h>
@@ -310,6 +314,8 @@ int test_ls_pp(librbd::RBD& rbd, librados::IoCtx& io_ctx, size_t num_expected, .
     assert(listed_name != names.end());
     names.erase(listed_name);
   }
+  va_end(ap);
+
   assert(names.empty());
 
   return num;
@@ -428,10 +434,9 @@ TEST(LibRBD, TestCopyPP)
 
 int test_ls_snaps(rbd_image_t image, int num_expected, ...)
 {
-  rbd_snap_info_t *snaps;
   int num_snaps, i, j, max_size = 10;
   va_list ap;
-  snaps = (rbd_snap_info_t *) malloc(sizeof(rbd_snap_info_t *) * max_size);
+  rbd_snap_info_t snaps[max_size];
   num_snaps = rbd_snap_list(image, snaps, &max_size);
   printf("num snaps is: %d\nexpected: %d\n", num_snaps, num_expected);
 
@@ -463,7 +468,6 @@ int test_ls_snaps(rbd_image_t image, int num_expected, ...)
   for (i = 0; i < num_snaps; i++) {
     assert(snaps[i].name == NULL);
   }
-  free(snaps);
 
   return num_snaps;
 }
@@ -1772,3 +1776,16 @@ TEST(LibRBD, DiffIterateStress)
   ioctx.close();
   ASSERT_EQ(0, destroy_one_pool_pp(pool_name, rados));
 }
+
+int main(int argc, char **argv)
+{
+  ::testing::InitGoogleTest(&argc, argv);
+
+  vector<const char*> args;
+  argv_to_vec(argc, (const char **)argv, args);
+
+  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
+  common_init_finish(g_ceph_context);
+
+  return RUN_ALL_TESTS();
+}
