@@ -11,6 +11,16 @@
  * Foundation.  See file COPYING.
  * 
  */
+#include "acconfig.h"
+
+#ifdef HAVE_SYS_MOUNT_H
+#include <sys/mount.h>
+#endif
+
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+
 #include <iostream>
 #include <string>
 #include <map>
@@ -247,7 +257,7 @@ class ClientStub : public TestStub
 	    << messenger->get_myaddr() << dendl;
 
     objecter.reset(new Objecter(cct, messenger.get(), &monc, &osdmap,
-				lock, timer));
+				lock, timer, 0, 0));
     assert(objecter.get() != NULL);
     objecter->set_balanced_budget();
 
@@ -366,8 +376,8 @@ class OSDStub : public TestStub
 
     messenger->set_default_policy(
 	Messenger::Policy::stateless_server(supported, 0));
-    messenger->set_policy_throttler(entity_name_t::TYPE_CLIENT,
-	&throttler);
+    messenger->set_policy_throttlers(entity_name_t::TYPE_CLIENT,
+				    &throttler, NULL);
     messenger->set_policy(entity_name_t::TYPE_MON,
 	Messenger::Policy::lossy_client(supported, CEPH_FEATURE_UID |
 	  CEPH_FEATURE_PGID64 |
@@ -521,7 +531,13 @@ class OSDStub : public TestStub
 
   void update_osd_stat() {
     struct statfs stbuf;
-    statfs(".", &stbuf);
+    int ret = statfs(".", &stbuf);
+    if (ret < 0) {
+      ret = -errno;
+      dout(0) << __func__ 
+              << " cannot statfs ." << cpp_strerror(ret) << dendl;
+      return;
+    }
 
     osd_stat.kb = stbuf.f_blocks * stbuf.f_bsize / 1024;
     osd_stat.kb_used = (stbuf.f_blocks - stbuf.f_bfree) * stbuf.f_bsize / 1024;
@@ -577,7 +593,7 @@ class OSDStub : public TestStub
     if (s.state & PG_STATE_CLEAN)
       s.last_clean = now;
     s.last_change = now;
-    s.reported.inc(1);
+    s.reported_seq++;
 
     pgs_changes.insert(pgid);
   }
@@ -994,9 +1010,6 @@ int main(int argc, const char *argv[])
   common_init_finish(g_ceph_context);
   g_ceph_context->_conf->apply_changes(NULL);
 
-  std::string command;
-  std::vector<std::string> command_args;
-
   set<int> stub_ids;
   double duration = 300.0;
 
