@@ -22,10 +22,11 @@ public:
 		    bufferlist *pbl, uint64_t trunc_size,  __u32 trunc_seq,
 		    Context *onfinish);
 
-  virtual tid_t write(const object_t& oid, const object_locator_t& oloc,
-		      uint64_t off, uint64_t len, const SnapContext& snapc,
-		      const bufferlist &bl, utime_t mtime, uint64_t trunc_size,
-		      __u32 trunc_seq, Context *oncommit);
+  virtual ceph_tid_t write(const object_t& oid, const object_locator_t& oloc,
+		           uint64_t off, uint64_t len,
+			   const SnapContext& snapc, const bufferlist &bl,
+			   utime_t mtime, uint64_t trunc_size,
+			   __u32 trunc_seq, Context *oncommit);
 
   virtual bool may_copy_on_write(const object_t&, uint64_t, uint64_t, snapid_t);
 private:
