@@ -261,10 +261,12 @@ class RESTDispatcher {
   } req_wq;
 
 public:
-  RESTDispatcher(CephContext *cct, int num_threads)
-    : m_tp(cct, "RESTDispatcher::m_tp", num_threads),
-      req_wq(this, g_conf->rgw_op_thread_timeout,
-	     g_conf->rgw_op_thread_suicide_timeout, &m_tp) {
+  CephContext *cct;
+  RESTDispatcher(CephContext *cct_, int num_threads)
+    : m_tp(cct_, "RESTDispatcher::m_tp", num_threads),
+      req_wq(this, cct_->_conf->rgw_op_thread_timeout,
+        cct_->_conf->rgw_op_thread_suicide_timeout, &m_tp),
+      cct(cct_) {
 
 
     response_handler.propertiesCallback = properties_callback;
@@ -588,6 +590,7 @@ protected:
 
 public:
   RESTBencher(RESTDispatcher *_dispatcher) :
+      ObjBencher(_dispatcher->cct),
       dispatcher(_dispatcher),
       completions(NULL),
       list_start(NULL),
@@ -659,7 +662,6 @@ int main(int argc, const char **argv)
   global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
   common_init_finish(g_ceph_context);
 
-  std::map < std::string, std::string > opts;
   std::vector<const char*>::iterator i;
   std::string host;
   std::string val;
@@ -727,7 +729,7 @@ int main(int argc, const char **argv)
     } else {
       if (val[0] == '-')
         usage_exit();
-      i++;
+      ++i;
     }
   }
 
@@ -786,7 +788,8 @@ int main(int argc, const char **argv)
     if (ret != 0)
       cerr << "error during cleanup: " << ret << std::endl;
   } else {
-    ret = bencher.aio_bench(operation, seconds, concurrent_ios, op_size, cleanup);
+    ret = bencher.aio_bench(operation, seconds, 0,
+			    concurrent_ios, op_size, cleanup);
     if (ret != 0) {
         cerr << "error during benchmark: " << ret << std::endl;
     }
